﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;

namespace ORM05
{
    class DbContext
    {
        private DbProvider dbProvider;
        private DbSqlBuilder sqlBuilder;

        public DbContext(string connectionString)
        {
            SqlConnection conn = new SqlConnection(connectionString);
            dbProvider = new DbProvider(conn);
            this.sqlBuilder = new DbSqlBuilder();
        }

        public int Insert<T>(object entity)
        {
            var entityMapping = AttributeMapping.Get<T>();

            //将Entity转换成Dictionary
            var parameters = DynamicMethodBuilder.ConvertFromObject(entity);
            var sql = sqlBuilder.BuildInsertSql(entityMapping, parameters.Keys.ToList());
            return dbProvider.ExecuteNonQuery(sql, parameters);
        }

        public int Update<T>(T entity)
        {
            var entityMapping = AttributeMapping.Get<T>();
            //将Entity转换成Dictionary
            var parameters = DynamicMethodBuilder.ConvertFromObject(entity);
            var columns = entityMapping.Members.Where(m => m.IsDbGenerated == false && m.IsPrimaryKey == false).Select(c => c.ColumnName).ToArray();
            var updateColumns = new Dictionary<string, object>();
            var whereColumns = new Dictionary<string, object>();

            foreach (var item in parameters)
            {
                if (columns.Contains(item.Key))
                    updateColumns.Add(item.Key, item.Value);
                if (entityMapping.PrimaryKey.All(m => m.ColumnName == item.Key))
                    whereColumns.Add(item.Key, item.Value);
            }

            var sql = sqlBuilder.BuildUpdateSql(entityMapping, updateColumns.Keys.ToList(), whereColumns.Keys.ToList());

            return dbProvider.ExecuteNonQuery(sql, parameters);
        }

        public int DeleteByKey<T>(params object[] values)
        {
            var entityMapping = AttributeMapping.Get<T>();

            if (values.Length != entityMapping.PrimaryKey.Count)
                throw new ArgumentException("参数个数和主键数不一致");

            var parameters = new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);
            for (int i = 0; i < entityMapping.PrimaryKey.Count; i++)
            {
                parameters.Add(entityMapping.PrimaryKey[i].ColumnName, values[i]);
            }

            var sql = sqlBuilder.BuildDeleteSql(entityMapping, parameters.Keys.ToList());

            return dbProvider.ExecuteNonQuery(sql, parameters);
        }

        public T Get<T>(object[] values)
        {
            var entityMapping = AttributeMapping.Get<T>();

            if (values.Length != entityMapping.PrimaryKey.Count)
                throw new ArgumentException("参数个数和主键数不一致");

            var parameters = new Dictionary<string, object>(StringComparer.OrdinalIgnoreCase);

            StringBuilder where = new StringBuilder();

            for (int i = 0; i < values.Length; i++)
            {
                if (i > 0) //考虑到有多个主键
                    where.Append(" AND ");

                where.Append(entityMapping.PrimaryKey[i].ColumnName).Append("=").Append("@p").Append(i);

                /*参数*/
                parameters.Add("@p" + i, values[i]);
            }

            var sql = this.sqlBuilder.BuildSelectSql(entityMapping, where.ToString(), string.Empty);

            return GetEntityList<T>(sql, parameters).FirstOrDefault();
        }

        public List<T> GetList<T>(string where, string orderBy)
        {
            var entityMapping = AttributeMapping.Get<T>();

            var sql = this.sqlBuilder.BuildSelectSql(entityMapping, where.ToString(), orderBy);

            Console.WriteLine(sql);
            return GetEntityList<T>(sql, null);
        }

        public List<T> GetEntityList<T>(string sql, Dictionary<string, object> parameters)
        {
            var entityMapping = AttributeMapping.Get<T>();

            using (var reader = dbProvider.ExecuteReader(sql, parameters))
            {
                List<T> list = new List<T>();

                var func = DynamicMethodBuilder.GetTypeDeserializer<T>(entityMapping);
                DbFieldReader fr = new DbFieldReader(reader);

                while (reader.Read())
                {
                    var entity = func(fr);
                    list.Add(entity);
                }
                return list;
            }
        }
    }
}